/*  diggstream: real-time activity from digg.com
    Andrew Hedges, andrew@hedges.name
    http://andrew.hedges.name
*/

var itms, len, cont, tmpl, evt, rstrt, flippie, donebutton

/*  Simple HTML templates in pure JavaScript
    Andrew Hedges, andrew@hedges.name
    http://snipt.org/rpp
*/
function Template(str) {
    this.rgx = /\{([^{}]*)\}/g
    this.str = str || ''
}

Template.prototype.evaluate = function (vals) {
    var repr
    vals = vals || {}
    repr = function (str, match) {
        return 'string' === typeof vals[match] || 'number' === typeof vals[match] ? vals[match] : str
    }
    return this.str.replace(this.rgx, repr)
}

itms = []
len  = 10
cont = document.getElementById('stream')
tmpl = new Template(
    '<li>' +
        '<a class="count" data-href="{href}" title="{title}">{count}</a>' +
        '<a class="title" data-href="{href}" title="{title}">{title}</a>' +
        '<div class="clearer"></div>' +
    '</li>'
)

function insert(item) {
    var key, i, len
    key = item.id
    for (i = 0, len = itms.length; i < len; ++i) {
        if (key === itms[i].id) {
            itms[i] = item
            return
        }
    }
    itms[itms.length] = item
}

function compare(a, b) {
    if (a.diggs > b.diggs)
        return -1
    if (a.diggs < b.diggs)
        return 1
    return 0
}

function sort() {
    if (itms.length > 1)
        itms.sort(compare)
    if (itms.length > len)
        itms.length = len
}

function rnd(diggs) {
    if (diggs > 999)
        diggs = Math.round(diggs / 100) / 10 + 'K'
    return diggs
}

function draw() {
    var html, i, len
    html = ''
    for (i = 0, len = itms.length; i < len; ++i) {
        html += tmpl.evaluate(itms[i])
    }
    cont.innerHTML = html
}

function start() {
    evt        = new EventSource('http://services.digg.com/2.0/stream?types=digg&format=event-stream')
    evt.onopen = function (e) {
        clearTimeout(rstrt)
    }
    evt.onerror = function (e) {
        rstrt = setTimeout(start, 5000)
    }
    evt.onmessage = function (e) {
        var data, sel, li
        data = JSON.parse(e.data)
        if (data.item) {
            data.item.count = rnd(data.item.diggs)
            insert(data.item)
            sort()
            draw()
        }
    }
}

function stop() {
    clearTimeout(rstrt)
    if (evt && evt.close) {
        evt.close()
    }
}

function startover() {
    stop()
    itms = []
    draw()
    start()
}

if (!window.widget)
    window.widget = {
        openURL : function (href) {
            window.open(href, 'digg')
        },
        prepareForTransition : function () {},
        performTransition : function () {}
    }

// flip widget over
function showBack() {
    clearTimeout(rstrt)
    window.widget.prepareForTransition('ToBack')
    document.getElementById('front').style.display = 'none'
    document.getElementById('back').style.display  = 'block'
    rstrt = setTimeout(function () {
        stop()
        window.widget.performTransition()
    }, 0)
}

// flip widget back
function showFront() {
    clearTimeout(rstrt)
    window.widget.prepareForTransition('ToFront')
    document.getElementById('front').style.display = 'block'
    document.getElementById('back').style.display  = 'none'
    rstrt = setTimeout(function () {
        start()
        window.widget.performTransition()
    }, 0)
}

// set up event handlers
document.getElementById('startover').addEventListener('click', startover, false)
document.getElementById('stream').addEventListener('mouseover', stop, false)
document.getElementById('stream').addEventListener('mouseout', start, false)

document.querySelector('body').addEventListener('click', function (e) {
    if ('A' === e.target.tagName)
        window.widget.openURL(e.target.getAttribute('data-href'))
    return false
}, false)

window.widget.onshow = start
window.widget.onhide = stop

// create buttons
flippie    = new AppleInfoButton(document.getElementById('flippie'), document.getElementById('front'), 'white', 'white', showBack)
donebutton = new AppleGlassButton(document.getElementById('donebutton'), 'Done', showFront)
